@@ -11,6 +11,7 @@
 
 #include "xeus/xkernel.hpp"
 #include "xeus/xkernel_configuration.hpp"
+#include "xeus/xserver_zmq.hpp"
 
 #include "custom_interpreter.hpp"
 
@@ -20,12 +21,14 @@ int main(int argc, char* argv[])
     std::string file_name = (argc == 1) ? "connection.json" : argv[2];
     xeus::xconfiguration config = xeus::load_configuration(file_name);
 
+    auto context = xeus::make_context<zmq::context_t>();
+
     // Create interpreter instance
     using interpreter_ptr = std::unique_ptr<custom::custom_interpreter>;
     interpreter_ptr interpreter = interpreter_ptr(new custom::custom_interpreter());
 
     // Create kernel instance and start it
-    xeus::xkernel kernel(config, xeus::get_user_name(), std::move(interpreter));
+    xeus::xkernel kernel(config, xeus::get_user_name(), std::move(context), std::move(interpreter), xeus::make_xserver_zmq);
     kernel.start();
 
     return 0;
