@@ -37,9 +37,10 @@
 
 namespace xeus
 {
-    std::string get_user_name()
-    {
-#if (defined(LINUX_PLATFORM) || defined(APPLE_PLATFORM))
+    std::string get_user_name(){
+#if defined(XEUS_EMSCRIPTEN_WASM_BUILD)
+    return "unspecified user";
+#elif (defined(LINUX_PLATFORM) || defined(APPLE_PLATFORM))
         struct passwd* pws;
         pws = getpwuid(geteuid());
         if (pws != nullptr)
@@ -72,51 +73,51 @@ namespace xeus
 
     xkernel::xkernel(const xconfiguration& config,
                      const std::string& user_name,
+                     context_ptr context,
                      interpreter_ptr interpreter,
+                     server_builder sbuilder,
                      history_manager_ptr history_manager,
                      logger_ptr logger,
-                     server_builder sbuilder,
                      debugger_builder dbuilder,
                      nl::json debugger_config,
                      nl::json::error_handler_t eh)
         : m_config(config)
         , m_user_name(user_name)
+        , p_context(std::move(context))
         , p_interpreter(std::move(interpreter))
         , p_history_manager(std::move(history_manager))
         , p_logger(std::move(logger))
-        , m_server_builder(sbuilder)
-        , m_debugger_builder(dbuilder)
         , m_debugger_config(debugger_config)
         , m_error_handler(eh)
     {
-        init();
+        init(sbuilder, dbuilder);
     }
 
     xkernel::xkernel(const std::string& user_name,
+                     context_ptr context,
                      interpreter_ptr interpreter,
+                     server_builder sbuilder,
                      history_manager_ptr history_manager,
                      logger_ptr logger,
-                     server_builder sbuilder,
                      debugger_builder dbuilder,
                      nl::json debugger_config,
                      nl::json::error_handler_t eh)
         : m_user_name(user_name)
+        , p_context(std::move(context))
         , p_interpreter(std::move(interpreter))
         , p_history_manager(std::move(history_manager))
         , p_logger(std::move(logger))
-        , m_server_builder(sbuilder)
-        , m_debugger_builder(dbuilder)
         , m_debugger_config(debugger_config)
         , m_error_handler(eh)
     {
-        init();
+        init(sbuilder, dbuilder);
     }
 
     xkernel::~xkernel()
     {
     }
 
-    void xkernel::init()
+    void xkernel::init(server_builder sbuilder, debugger_builder dbuilder)
     {
         m_kernel_id = new_xguid();
         m_session_id = new_xguid();
@@ -126,23 +127,19 @@ namespace xeus
             m_config.m_key = new_xguid();
         }
 
-        using authentication_ptr = xkernel_core::authentication_ptr;
-        authentication_ptr auth = make_xauthentication(m_config.m_signature_scheme, m_config.m_key);
-
         if(p_logger == nullptr || std::getenv("XEUS_LOG") == nullptr)
         {
             p_logger = std::make_unique<xlogger_nolog>();
         }
 
-        p_server = m_server_builder(m_context, m_config);
+        p_server = sbuilder(*p_context, m_config, m_error_handler);
         p_server->update_config(m_config);
 
-        p_debugger = m_debugger_builder(m_context, m_config, m_user_name, m_session_id, m_debugger_config);
+        p_debugger = dbuilder(*p_context, m_config, m_user_name, m_session_id, m_debugger_config);
 
         p_core = std::make_unique<xkernel_core>(m_kernel_id,
                                                 m_user_name,
                                                 m_session_id,
-                                                std::move(auth),
                                                 p_logger.get(),
                                                 p_server.get(),
                                                 p_interpreter.get(),
@@ -164,8 +161,14 @@ namespace xeus
 
     void xkernel::start()
     {
-        zmq::multipart_t start_msg = p_core->build_start_msg();
-        p_server->start(start_msg);
+        xpub_message start_msg = p_core->build_start_msg();
+        p_server->start(std::move(start_msg));
+    }
+
+    void xkernel::stop()
+    {
+        p_interpreter->shutdown_request();
+        p_server->stop();
     }
 
     const xconfiguration& xkernel::get_config()
