@@ -26,7 +26,6 @@ namespace xeus
     xkernel_core::xkernel_core(const std::string& kernel_id,
                                const std::string& user_name,
                                const std::string& session_id,
-                               authentication_ptr auth,
                                logger_ptr logger,
                                server_ptr server,
                                interpreter_ptr interpreter,
@@ -36,7 +35,6 @@ namespace xeus
         : m_kernel_id(std::move(kernel_id))
         , m_user_name(std::move(user_name))
         , m_session_id(std::move(session_id))
-        , p_auth(std::move(auth))
         , m_comm_manager(this)
         , p_logger(logger)
         , p_server(server)
@@ -88,10 +86,8 @@ namespace xeus
     {
     }
 
-    zmq::multipart_t xkernel_core::build_start_msg() const
+    xpub_message xkernel_core::build_start_msg() const
     {
-        zmq::multipart_t start_msg;
-
         std::string topic = "kernel_core." + m_kernel_id + ".status";
         nl::json content;
         content["execution_state"] = "starting";
@@ -102,27 +98,23 @@ namespace xeus
                          nl::json::object(),
                          std::move(content),
                          buffer_sequence());
-        std::move(msg).serialize(start_msg, *p_auth, m_error_handler);
-
-        return start_msg;
+        return msg;
     }
 
-    void xkernel_core::dispatch_shell(zmq::multipart_t& wire_msg)
+    void xkernel_core::dispatch_shell(xmessage msg)
     {
-        dispatch(wire_msg, channel::SHELL);
+        dispatch(std::move(msg), channel::SHELL);
     }
 
-    void xkernel_core::dispatch_control(zmq::multipart_t& wire_msg)
+    void xkernel_core::dispatch_control(xmessage msg)
     {
-        dispatch(wire_msg, channel::CONTROL);
+        dispatch(std::move(msg), channel::CONTROL);
     }
 
-    void xkernel_core::dispatch_stdin(zmq::multipart_t& wire_msg)
+    void xkernel_core::dispatch_stdin(xmessage msg)
     {
-        xmessage msg;
         try
         {
-            msg.deserialize(wire_msg, *p_auth);
             p_logger->log_received_message(msg, xlogger::stdinput);
             const nl::json& content = msg.content();
             std::string value = content.value("value", "");
@@ -136,11 +128,10 @@ namespace xeus
         }
     }
 
-    zmq::multipart_t xkernel_core::dispatch_internal(zmq::multipart_t& wire_msg)
+    nl::json xkernel_core::dispatch_internal(nl::json msg)
     {
-        nl::json msg = nl::json::parse(wire_msg.popstr());
         nl::json rep = p_interpreter->internal_request(msg);
-        return zmq::multipart_t(rep.dump(-1, ' ', false, m_error_handler));
+        return rep;
     }
 
     void xkernel_core::publish_message(const std::string& msg_type,
@@ -149,8 +140,6 @@ namespace xeus
                                        buffer_sequence buffers,
                                        channel c)
     {
-
-        zmq::multipart_t wire_msg;
         xpub_message msg(get_topic(msg_type),
                          make_header(msg_type, m_user_name, m_session_id),
                          get_parent_header(c),
@@ -158,15 +147,13 @@ namespace xeus
                          std::move(content),
                          std::move(buffers));
         p_logger->log_iopub_message(msg);
-        std::move(msg).serialize(wire_msg, *p_auth, m_error_handler);
-        p_server->publish(wire_msg, c);
+        p_server->publish(std::move(msg), c);
     }
 
     void xkernel_core::send_stdin(const std::string& msg_type,
                                   nl::json metadata,
                                   nl::json content)
     {
-        zmq::multipart_t wire_msg;
         xmessage msg(get_parent_id(channel::SHELL),
                      make_header(msg_type, m_user_name, m_session_id),
                      get_parent_header(channel::SHELL),
@@ -174,8 +161,7 @@ namespace xeus
                      std::move(content),
                      buffer_sequence());
         p_logger->log_sent_message(msg, xlogger::stdinput);
-        std::move(msg).serialize(wire_msg, *p_auth, m_error_handler);
-        p_server->send_stdin(wire_msg);
+        p_server->send_stdin(std::move(msg));
     }
 
     xcomm_manager& xkernel_core::comm_manager() & noexcept
@@ -198,20 +184,8 @@ namespace xeus
         return m_parent_header[std::size_t(c)];
     }
 
-    void xkernel_core::dispatch(zmq::multipart_t& wire_msg, channel c)
+    void xkernel_core::dispatch(xmessage msg, channel c)
     {
-        xmessage msg;
-        try
-        {
-            msg.deserialize(wire_msg, *p_auth);
-        }
-        catch (std::exception& e)
-        {
-            std::cerr << "ERROR: could not deserialize message" << std::endl;
-            std::cerr << e.what() << std::endl;
-            return;
-        }
-
         p_logger->log_received_message(msg, c == channel::SHELL ? xlogger::shell : xlogger::control);
         const nl::json& header = msg.header();
         set_parent(msg.identities(), header, c);
@@ -222,17 +196,18 @@ namespace xeus
         if (handler == nullptr)
         {
             std::cerr << "ERROR: received unknown message" << std::endl;
+            std::cerr << "Message type: " << msg_type << std::endl;
         }
         else
         {
             try
             {
-                (this->*handler)(msg, c);
+                (this->*handler)(std::move(msg), c);
             }
             catch (std::exception& e)
             {
                 std::cerr << "ERROR: received bad message: " << e.what() << std::endl;
-                std::cerr << "Message content: " << msg.content() << std::endl;
+                std::cerr << "Message type: " << msg_type << std::endl;
             }
         }
 
@@ -246,7 +221,7 @@ namespace xeus
         return res;
     }
 
-    void xkernel_core::execute_request(const xmessage& request, channel c)
+    void xkernel_core::execute_request(xmessage request, channel c)
     {
         try
         {
@@ -286,7 +261,7 @@ namespace xeus
         }
     }
 
-    void xkernel_core::complete_request(const xmessage& request, channel c)
+    void xkernel_core::complete_request(xmessage request, channel c)
     {
         const nl::json& content = request.content();
         std::string code = content.value("code", "");
@@ -296,7 +271,7 @@ namespace xeus
         send_reply("complete_reply", nl::json::object(), std::move(reply), c);
     }
 
-    void xkernel_core::inspect_request(const xmessage& request, channel c)
+    void xkernel_core::inspect_request(xmessage request, channel c)
     {
         const nl::json& content = request.content();
         std::string code = content.value("code", "");
@@ -307,7 +282,7 @@ namespace xeus
         send_reply("inspect_reply", nl::json::object(), std::move(reply), c);
     }
 
-    void xkernel_core::history_request(const xmessage& request, channel c)
+    void xkernel_core::history_request(xmessage request, channel c)
     {
         const nl::json& content = request.content();
 
@@ -316,7 +291,7 @@ namespace xeus
         send_reply("history_reply", nl::json::object(), std::move(history), c);
     }
 
-    void xkernel_core::is_complete_request(const xmessage& request, channel c)
+    void xkernel_core::is_complete_request(xmessage request, channel c)
     {
         const nl::json& content = request.content();
         std::string code = content.value("code", "");
@@ -325,7 +300,7 @@ namespace xeus
         send_reply("is_complete_reply", nl::json::object(), std::move(reply), c);
     }
 
-    void xkernel_core::comm_info_request(const xmessage& request, channel c)
+    void xkernel_core::comm_info_request(xmessage request, channel c)
     {
         const nl::json& content = request.content();
         std::string target_name = content.is_null() ? "" : content.value("target_name", "");
@@ -346,14 +321,14 @@ namespace xeus
         send_reply("comm_info_reply", nl::json::object(), std::move(reply), c);
     }
 
-    void xkernel_core::kernel_info_request(const xmessage& /* request */, channel c)
+    void xkernel_core::kernel_info_request(xmessage /* request */, channel c)
     {
         nl::json reply = p_interpreter->kernel_info_request();
         reply["protocol_version"] = get_protocol_version();
         send_reply("kernel_info_reply", nl::json::object(), std::move(reply), c);
     }
 
-    void xkernel_core::shutdown_request(const xmessage& request, channel c)
+    void xkernel_core::shutdown_request(xmessage request, channel c)
     {
         const nl::json& content = request.content();
         bool restart = content.value("restart", false);
@@ -365,14 +340,14 @@ namespace xeus
         send_reply("shutdown_reply", nl::json::object(), std::move(reply), c);
     }
 
-    void xkernel_core::interrupt_request(const xmessage&, channel c)
+    void xkernel_core::interrupt_request(xmessage, channel c)
     {
         nl::json reply = nl::json::object();
         publish_message("interrupt", nl::json::object(), nl::json(reply), buffer_sequence(), channel::CONTROL);
         send_reply("interrupt_reply", nl::json::object(), std::move(reply), c);
     }
 
-    void xkernel_core::debug_request(const xmessage& request, channel c)
+    void xkernel_core::debug_request(xmessage request, channel c)
     {
         if(p_debugger)
         {
@@ -422,7 +397,6 @@ namespace xeus
                                   nl::json reply_content,
                                   channel c)
     {
-        zmq::multipart_t wire_msg;
         xmessage reply(id_list,
                        make_header(reply_type, m_user_name, m_session_id),
                        std::move(parent_header),
@@ -430,29 +404,18 @@ namespace xeus
                        std::move(reply_content),
                        buffer_sequence());
         p_logger->log_sent_message(reply, c == channel::SHELL ? xlogger::shell : xlogger::control);
-        std::move(reply).serialize(wire_msg, *p_auth, m_error_handler);
         if (c == channel::SHELL)
         {
-            p_server->send_shell(wire_msg);
+            p_server->send_shell(std::move(reply));
         }
         else
         {
-            p_server->send_control(wire_msg);
+            p_server->send_control(std::move(reply));
         }
     }
 
-    void xkernel_core::abort_request(zmq::multipart_t& wire_msg)
+    void xkernel_core::abort_request(xmessage msg)
     {
-        xmessage msg;
-        try
-        {
-            msg.deserialize(wire_msg, *p_auth);
-        }
-        catch (std::exception& e)
-        {
-            std::cerr << "ERROR: during execute_request: " << e.what() << std::endl;
-            return;
-        }
         const nl::json& header = msg.header();
         std::string msg_type = header.value("msg_type", "");
         // replace "_request" part of message type by "_reply"
@@ -498,18 +461,18 @@ namespace xeus
         return m_parent_header[std::size_t(c)];
     }
 
-    void xkernel_core::comm_open(const xmessage& request, channel)
+    void xkernel_core::comm_open(xmessage request, channel)
     {
-        return m_comm_manager.comm_open(request);
+        m_comm_manager.comm_open(std::move(request));
     }
 
-    void xkernel_core::comm_close(const xmessage& request, channel)
+    void xkernel_core::comm_close(xmessage request, channel)
     {
-        return m_comm_manager.comm_close(request);
+        m_comm_manager.comm_close(std::move(request));
     }
 
-    void xkernel_core::comm_msg(const xmessage& request, channel)
+    void xkernel_core::comm_msg(xmessage request, channel)
     {
-        return m_comm_manager.comm_msg(request);
+        m_comm_manager.comm_msg(std::move(request));
     }
 }
