@@ -10,17 +10,25 @@
 #include <memory>
 
 #include "test_interpreter.hpp"
+#include "xeus/xeus_context.hpp"
 #include "xeus/xkernel.hpp"
 #include "xeus/xkernel_configuration.hpp"
+#include "xeus/xserver_zmq.hpp"
 
 int main(int argc, char* argv[])
 {
     std::string file_name = (argc == 1) ? "connection.json" : argv[2];
     xeus::xconfiguration config = xeus::load_configuration(file_name);
 
+    auto context = xeus::make_context<zmq::context_t>();
+
     using interpreter_ptr = std::unique_ptr<test_kernel::test_interpreter>;
     interpreter_ptr interpreter = interpreter_ptr(new test_kernel::test_interpreter());
-    xeus::xkernel kernel(config, xeus::get_user_name(), std::move(interpreter));
+    xeus::xkernel kernel(config,
+                         xeus::get_user_name(),
+                         std::move(context),
+                         std::move(interpreter),
+                         xeus::make_xserver_zmq);
     std::cout << "starting kernel" << std::endl;
     kernel.start();
 
